<?php

require_once 'abstract.php';

/**
 * Class VerifoneGenerateUrlModuleFrontController
 */
class VerifoneGenerateUrlModuleFrontController extends VerifoneAbstarctModuleFrontController
{

    /* @var $method */
    protected $method;
    private $logger;

    public function postProcess()
    {

        $response = null;
        $this->logger = new Verifone_Cart_Logger(__FILE__);

        try {
            $module_name = Tools::getValue('module_name');

            if (!$module_name) {
                $msg = $this->l('Invalid module name in Generate Url controller');
                throw new PrestaShopException($msg);
            }
            $this->module = Module::getInstanceByName($module_name);

            /**
             * Get current cart object from session
             */
            $cart = $this->context->cart;
            $authorized = false;

            /**
             * Verify if this module is enabled and if the cart has
             * a valid customer, delivery address and invoice address
             */
            if (!$this->module->active || $cart->id_customer == 0 || $cart->id_address_delivery == 0
                || $cart->id_address_invoice == 0) {
                Tools::redirect('index.php?controller=order&step=1');
            }

            /**
             * Verify if this payment module is authorized
             */
            foreach (Module::getPaymentModules() as $module) {
                if ($module['name'] == 'verifone') {
                    $authorized = true;
                    break;
                }
            }

            if (!$authorized) {
                die($this->l('This payment method is not available.'));
            }

            /** @var CustomerCore $customer */
            $customer = new Customer($cart->id_customer);

            /**
             * Check if this is a valid customer account
             */
            if (!Validate::isLoadedObject($customer)) {
                Tools::redirect('index.php?controller=order&step=1');
            }

            /** @var $customer \Customer */
            $customer = new Customer($cart->id_customer);;

            if ($this->module->verifone_settings->is3dsEnabled() && null === $this->module->verifone_settings->get3dsContractId()) {
                $msg = sprintf($this->l('3DS is enabled but no 3DS contract ID is provided in the configuration.
            Please fill in the field "3DS Contract Id" or disable the field "Enable/Disable 3d-secure"
            in the plugin settings.'));
                throw new PrestaShopException($msg);
            }

            //Check if live / test accounts passwords are set
            if (!$this->module->verifone_settings->getApiUserId()
                || !$this->module->verifone_settings->getApiKey()) {
                $msg = sprintf($this->l('Api User OR Api Key is missing! Please check module configuration.'));
                throw new PrestaShopException($msg);
            }


            $verifone_customer = new Verifone_Customer(
                array('cart' => $cart, 'customer' => $customer),
                $this->module->verifone_settings);

            $result = $this->module->client->postCustomer($verifone_customer->toArray());

            $customer_id = $result['id'];
            $returnUrl = $this->context->link->getModuleLink($this->module->name, 'paymentCallback',
                [
                    'id_cart' => $cart->id,
                    'id_module' => (int)$this->module->id,
                    'key' => $customer->secure_key
                ],
                true);

            $verifone_checkout = new Verifone_Checkout(
                array('cart' => $cart,
                    'currency' => $this->context->currency,
                    'returnUrl' => $returnUrl
                ),
                $this->module->verifone_settings,
                $this->module->verifone_helper,
                $customer_id);


            $result = $this->module->client->postCheckout($verifone_checkout->toArray());

            if (!isset($result['url'])) {
                $msg = sprintf($this->l('The "url" key is not present in the verifone response,
            unable to continue, API response: %s'), json_encode($result));
                throw new PrestaShopException($msg);
            }

            //Success :) !
            $response = array('status' => 'success', 'redirect_link' => $result['url']);
        } catch (Exception $e) {
            $this->logger->error(sprintf($this->l('Error: %s'), $e->getMessage()), __LINE__);
            $this->errors['err'] = $this->l('We encountered an error, please check Prestashop logs.');
        }

        //Errors :( !
        if (!empty($this->errors)) {
            $response = array('success' => false, 'redirect_link' => Context::getContext()->link->getModuleLink
            ($this->name, 'error', $this->errors));
        }

        // Classic json response
        header('Content-Type: application/json');
        die(json_encode($response));
    }
}
