<?php
require_once 'abstract.php';


/**
 * Class VerifonePaymentCallbackModuleFrontController
 */
class VerifonePaymentCallbackModuleFrontController extends VerifoneAbstarctModuleFrontController
{

    /* @var $method */
    protected $method;

    public $errors;

    public $currentOrder;

    public $order;

    public $trns;

    private $logger;

    public function init()
    {
        parent::init();
        $this->request = Tools::getAllValues();

    }

    public function postProcess()
    {
        $this->logger = new Verifone_Cart_Logger(__FILE__);
        $params = $this->request;

        try {
            $module_id = $params['id_module'];
            $cart_id = $params['id_cart'];

            if (!$module_id || !$cart_id) {
                $msg = sprintf($this->l('Verifone Payment Callback - No module name found. Invalid!'));
                throw new PrestaShopException($msg);
            }
            $this->module = Module::getInstanceById($module_id);

            if (!isset($params['checkout_id'])) {
                $msg = sprintf($this->l(' Verifone Payment Callback - The checkoutId parameter could not be found, in order to 
            proceed it must exist. Current GET params: %s'), json_encode($params));
                throw new PrestaShopException($msg);
            }

            $cart = new Cart($cart_id);

            if (!$cart) {
                $msg = sprintf($this->l('Verifone Payment Callback - Invalid cart!'));
                throw new PrestaShopException($msg);
            }

            /** @var CustomerCore $customer */
            $customer = new Customer($cart->id_customer);


            if (!$customer) {
                $msg = sprintf($this->l('Verifone Payment Callback - Invalid customer!'));
                throw new PrestaShopException($msg);
            }


            $response = $this->module->client->getCheckout($params['checkout_id']);

            if (!isset($response['events'])
                || !isset($response['merchant_reference'])
                || !isset($response['transaction_id'])
            ) {
                $msg = sprintf($this->l('The events or merchantReference or transactionId 
                parameter could not be found in response, in order to proceed it must exist. API response %s'),
                    json_encode($response));
                throw new PrestaShopException($msg);
            }

            //Check if live / test accounts passwords are set
            if (!$this->module->verifone_settings->getApiUserId()
                || !$this->module->verifone_settings->getApiKey()) {
                $msg = sprintf($this->l('Api User OR Api Key is missing! Please check module configuration.'));
                throw new PrestaShopException($msg);
            }


            if (!$this->errors) {
                $payment_currency_id = Currency::getIdByIsoCode($response['currency_code']);
                $payment_currency = Currency::getCurrencyInstance($payment_currency_id);
                $payment_currency_precision = $payment_currency->precision;

                //Now generate Order!
                $amount_paid = $this->transformFromCents($response['amount'], $payment_currency_precision);

                $this->order = $this->validateVerifoneOrder(
                    $cart->id,
                    Configuration::get('PS_OS_PREPARATION'),
                    $amount_paid,
                    $this->name,
                    'Order Created',
                    array(),
                    null,
                    false,
                    $params['key'],
                    null,
                    $payment_currency_precision
                );

                $this->redirectUrl = 'index.php?controller=order-confirmation&id_cart=' . $cart->id . '&id_module='
                    . $this->module->id . '&id_order=' . $this->order->id . '&key=' . $customer->secure_key;

                $history = new OrderHistory();
                $history->id_order = (int)$this->order->id;

                if (!$this->order->current_state) {
                    $this->logger->log(sprintf($this->l('Changing order "%s" status to "Processing"'),
                        $this->order->reference),
                        __LINE__);
                    $history->changeIdOrderState((int)Configuration::get('PS_OS_PREPARATION'), $this->order, true);
                    $history->save();
                }

                // set the order status based on the checkout response
                // i was only able to get 2 statuses to get pushed back
                foreach ($response['events'] as $event) {
                    if ('TRANSACTION_SUCCESS' == $event['type']) {
                        //save REFNO to payment
                        $orderPayment = OrderPayment::getByOrderReference($this->order->reference);
                        foreach ($orderPayment as $payment) {
                            $invoice = $payment->getOrderInvoice($this->order->id);
                            if ($invoice && !$payment->transaction_id && $this->order->current_state != _PS_OS_PAYMENT_) {
                                $payment->transaction_id = $response['transaction_id'];
                                $payment->save();
                                $this->logger->log(sprintf($this->l('Changing order "%s" status to "Payment accepted"'),
                                    $this->order->reference),
                                    __LINE__);
                                $history->changeIdOrderState(_PS_OS_PAYMENT_, $this->order, true);
                                $history->save();
                            }
                        }
                    }

                    if ('TRANSACTION_FAILED' == $event['type']) {
                        $logger_msg = sprintf($this->l('Payment FAILED for order: %s. Received response: %s '),
                            $this->order->reference, json_encode($response));
                        $this->logger->error($logger_msg, __LINE__);
                        $history->changeIdOrderState(_PS_OS_ERROR_, $this->order, true);
                        $history->save();
                        $msg = sprintf($this->l('Payment FAILED for order: %s.'), $this->order->reference);
                        throw new PrestaShopException($msg);
                    }

                    if ('TRANSACTION_DECLINED' == $event['type']) {
                        $logger_msg = sprintf($this->l('Payment DECLINED for order: %s. Received response: %s '),
                            $this->order->reference, json_encode($response));
                        $this->logger->error($logger_msg, __LINE__);
                        $history->changeIdOrderState(_PS_OS_ERROR_, $this->order, true);
                        $history->save();
                        $msg = sprintf($this->l('Payment DECLINED for order: %s.'), $this->order->reference);
                        throw new PrestaShopException($msg);
                    }
                }
            }
            $this->order->save();

        } catch (Exception $e) {
            $this->errors['err'] = sprintf($this->l('Payment failed for order %s, please try again or contact the merchant!'), $this->order->reference);
            $this->logger->error(sprintf($this->l('Exception handling payment callback: %s'),
                $e->getMessage()), __LINE__);
            $this->redirectUrl = Context::getContext()->link->getModuleLink($this->name, 'error', $this->errors);
        }
    }

    public function validateVerifoneOrder(
        $id_cart,
        $id_order_state,
        $amount_paid,
        $payment_method = 'Unknown',
        $message = null,
        $transaction = array(),
        $currency_special = null,
        $dont_touch_amount = false,
        $secure_key = false,
        Shop $shop = null,
        $precision = 2
    )
    {
        if ($this->needConvert()) {
            $amount_paid_curr = Tools::ps_round(Tools::convertPrice($amount_paid, new Currency($currency_special),
                true), $precision);
        } else {
            $amount_paid_curr = Tools::ps_round($amount_paid, $precision);
        }
        $amount_paid = Tools::ps_round($amount_paid, $precision);

        $cart = new Cart((int)$id_cart);
        $total_ps = (float)$cart->getOrderTotal(true, Cart::BOTH);
        if ($amount_paid_curr > $total_ps + 0.10 || $amount_paid_curr < $total_ps - 0.10) {
            $total_ps = $amount_paid_curr;
        }

        try {
            $this->module->validateOrder(
                (int)$id_cart,
                (int)$id_order_state,
                (float)$total_ps,
                $payment_method,
                $message,
                array('transaction_id' => isset($transaction['transaction_id']) ? $transaction['transaction_id'] : ''),
                $currency_special,
                $dont_touch_amount,
                $secure_key
            //$shop
            );
        } catch (Exception $e) {


            $msg = sprintf($this->l('Order validation error : "%s"; File: "%s"; Line: "%s";'), $e->getMessage(),
                $e->getFile(), $e->getLine());
            $this->logger->error($msg, __LINE__);

            $this->currentOrder = (int)Order::getIdByCartId((int)$id_cart);

            if ($this->currentOrder == false) {
                $msg = sprintf($this->l('Order validation error : "%s"'), $e->getMessage());
            }
            throw new PrestaShopException($msg);
        }

        $order = new Order($this->module->currentOrder);

        if (isset($amount_paid_curr) && $amount_paid_curr != 0 && $order->total_paid != $amount_paid_curr && $this->isOneOrder($order->reference)) {
            $order->total_paid = $amount_paid_curr;
            $order->total_paid_real = $amount_paid_curr;
            $order->total_paid_tax_incl = $amount_paid_curr;
            $order->update();

            $sql = 'UPDATE `' . _DB_PREFIX_ . 'order_payment`
		    SET `amount` = ' . (float)$amount_paid_curr . '
		    WHERE  `order_reference` = "' . pSQL($order->reference) . '"';
            Db::getInstance()->execute($sql);
        }

        return $order;
    }

    /**
     * Check if we need convert currency
     * @return boolean|integer currency id
     */
    public function needConvert()
    {
        $currency_mode = Currency::getPaymentCurrenciesSpecial($this->module->id);
        $mode_id = $currency_mode['id_currency'];
        if ($mode_id == -2) {
            return (int)Configuration::get('PS_CURRENCY_DEFAULT');
        } elseif ($mode_id == -1) {
            return false;
        } elseif ($mode_id != $this->context->currency->id) {
            return (int)$mode_id;
        } else {
            return false;
        }
    }

    public function isOneOrder($order_reference)
    {
        $query = new DBQuery();
        $query->select('COUNT(*)');
        $query->from('orders');
        $query->where('reference = "' . pSQL($order_reference) . '"');
        $countOrders = (int)DB::getInstance()->getValue($query);

        return $countOrders == 1;
    }

}
