<?php

class Verifone_Refund extends Verifone_Api_Entity {

	/**
	 * @var \Order
	 */
	private $order;

	/**
	 * @var array
	 */
	protected $refund_params;
	private $amount;
	private $reason;
	private $transaction_id;

	public function __construct(
		$amount,
		$reason, //or comment
        $transaction_id,
		Order $order
	) {
		$this->order    = $order;
		$this->amount   = $amount;
		$this->reason  = $reason;
        $this->transaction_id = $transaction_id;
		$this->buildData();
	}

	/**
	 * @throws \Exception
	 */
	private function buildData() {
		$created_at = $this->order->date_add;
        $format = 'Y-m-d H:i:s';
        $createdDateTime = DateTime::createFromFormat($format, $created_at);

		if(!$createdDateTime instanceof DateTime) {
			throw new Exception('There was an error in the application, please contact the site administrator');
		}

		$this->refund_params = [
            'id' => "{$this->transaction_id}",
            'reference_id' => "{$this->order->id_cart}",
			'amount' => (float)$this->amount * 100,
            'reason' => (string)$this->reason,
			'created_date_time' => "{$createdDateTime->format( 'Y-m-d\TH:i:s\Z' )}"
		];
	}

	/**
	 * @return array
	 */
	public function toArray()
	{
		return $this->refund_params;
	}

	/**
	 * @return false|string
	 */
	public function toJson() {
		return json_encode($this->refund_params);
	}

}
