<?php

require_once 'libs/interfaces/interface-verifone-hosted-arrayable.php';
require_once 'libs/interfaces/interface-verifone-hosted-jsonable.php';
require_once 'libs/abstract/abstract-verifone-api-entity.php';

require_once 'libs/includes/class-verifone-hosted-logger.php';
require_once 'libs/includes/class-verifone-hosted-checkout.php';
require_once 'libs/includes/class-verifone-hosted-helper.php';
require_once 'libs/includes/class-verifone-hosted-customer.php';
require_once 'libs/includes/class-verifone-hosted-helper.php';
require_once 'libs/includes/class-verifone-refund.php';

require_once 'libs/VerifoneEcomAPI/autoload.php';

use VerifoneEcomAPI\ApiWrapper\Authentication\BasicAuth;
use VerifoneEcomAPI\ApiWrapper\Http\ApiClient\Client;
use VerifoneEcomAPI\ApiWrapper\Http\SimpleCurl;
use VerifoneEcomAPI\ApiWrapper\Regions\Emea;
use VerifoneEcomAPI\ApiWrapper\Regions\Na;
use VerifoneEcomAPI\ApiWrapper\Regions\NzAu;
use VerifoneEcomAPI\ApiWrapper\Settings;


use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

/**
 * Class verifone - safe payment method
 */
class Verifone extends \PaymentModule
{
    const DEBUG_MODE = false;
    const REFUND_REASON = 'Other';
    public $tcoApi;
    public $details;
    public $owner;
    public $name;
    public $address;
    public $bootstrap;
    public $is_eu_compatible;
    public $extra_mail_vars;
    public $confirmUninstall;
    protected $_html = '';
    protected $_postErrors = [];

    /**
     * @var Client
     */
    public $client;

    /**
     * @var \Verifone_Helper
     */
    public $verifone_helper;

    /**
     * @var Settings
     */
    public $verifone_settings;

    /**
     * @var \Verifone_Cart_Logger
     */
    public $logger;

    /**
     * @var Emea|Na|NzAu
     */
    public $mapper;

    /**
     * @var SimpleCurl
     */
    public $simple_curl;

    /**
     * @var BasicAuth
     */
    public $auth;

    /**
     * @var \Verifone
     */
    private $module;

    /**
     * verifone constructor.
     */
    public function __construct()
    {
        $this->name = 'verifone';
        $this->tab = 'payments_gateways';
        $this->version = '2.1.1';
        $this->ps_versions_compliancy = ['min' => '1.7', 'max' => _PS_VERSION_];
        $this->author = 'Verifone';
        $this->controllers = ['validation'];
        $this->is_eu_compatible = 1;
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $settings = new Settings();
        $settings->setTest(Configuration::get('VERIFONE_TEST'))
            ->setRegion(Configuration::get('VERIFONE_REGION'))
            ->setApiUserId(Configuration::get('VERIFONE_API_USER'))
            ->setApiKey(base64_decode(Configuration::get('VERIFONE_API_KEY')))
            ->setTestApiUserId(Configuration::get('VERIFONE_TEST_API_USER'))
            ->setTestApiKey(base64_decode(Configuration::get('VERIFONE_TEST_API_KEY')))
            ->set3dsEnable(Configuration::get('VERIFONE_3DS_ENABLE'))
            ->setOverlayOnPlaceOrder(Configuration::get('VERIFONE_OVERLAY_ON_PLACE_ORDER'))
            ->set3dsContractId(Configuration::get('VERIFONE_3DS_CONTRACT_ID'))
            ->setPaymentContractId(Configuration::get('VERIFONE_PAYMENT_CONTRACT_ID'))
            ->setEntityId(Configuration::get('VERIFONE_ENTITY_ID'))
            ->setTestEntityId(Configuration::get('VERIFONE_TEST_ENTITY_ID'))
            ->setTestPaymentContractId(Configuration::get('VERIFONE_TEST_PAYMENT_CONTRACT_ID'))
            ->setTest3dsContractId(Configuration::get('VERIFONE_TEST_3DS_CONTRACT_ID'))
            ->setCurlVerifySsl(Configuration::get('VERIFONE_CURL_VERIFY_SSL'))
            ->setCvvOnPayment(Configuration::get('VERIFONE_CVV_ON_PAYMENT'));

        $this->verifone_settings = $settings;
        $this->logger = new Verifone_Cart_Logger(__FILE__);
        $this->simple_curl = new SimpleCurl();

        // mini factory :D
        switch (Configuration::get('VERIFONE_REGION')) {
            case 'NZAU':
                $this->mapper = new NzAu($this->verifone_settings);
                break;
            case 'NA':
                $this->mapper = new Na($this->verifone_settings);
                break;
            case 'EMEA':
                $this->mapper = new Emea($this->verifone_settings);
                break;
            default:
                // will happen only on the first installation
                $this->mapper = new NzAu($this->verifone_settings);
        }

        $this->auth = new BasicAuth($this->verifone_settings);
        $this->client = new Client(
            $this->mapper,
            $this->verifone_settings,
            $this->auth,
            $this->simple_curl
        );

        $this->verifone_helper = new Verifone_Helper();

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('Verifone');
        $this->description = $this->l('Verifone payment solutions');

        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall Verifone payment modules?');
    }

    /**
     * install the module
     * @return bool|string
     */
    public function install()
    {

        if (parent::install()
            && $this->registerHook('displayHeader')
            && $this->registerHook('paymentOptions')
            && $this->registerHook('displayOrderConfirmation')
            && $this->registerHook('actionProductCancel')
            && $this->registerHook('displayAdminOrder')
            && $this->registerHook('displayAdminOrderTop')
        ) {
            return true;
        }

        return false;
    }

    /**
     * uninstall the module and deletes the config keys
     * @return bool
     */
    function uninstall()
    {
        Configuration::deleteByName('VERIFONE_TITLE');
        Configuration::deleteByName('VERIFONE_DESCRIPTION');
        Configuration::deleteByName('VERIFONE_REGION');
        Configuration::deleteByName('VERIFONE_API_USER');
        Configuration::deleteByName('VERIFONE_API_KEY');
        Configuration::deleteByName('VERIFONE_ENTITY_ID');
        Configuration::deleteByName('VERIFONE_PAYMENT_CONTRACT_ID');
        Configuration::deleteByName('VERIFONE_3DS_CONTRACT_ID');
        Configuration::deleteByName('VERIFONE_TEST');
        Configuration::deleteByName('VERIFONE_TEST_API_USER');
        Configuration::deleteByName('VERIFONE_TEST_API_KEY');
        Configuration::deleteByName('VERIFONE_TEST_ENTITY_ID');
        Configuration::deleteByName('VERIFONE_TEST_PAYMENT_CONTRACT_ID');
        Configuration::deleteByName('VERIFONE_TEST_3DS_CONTRACT_ID');
        Configuration::deleteByName('VERIFONE_3DS_ENABLE');
        Configuration::deleteByName('VERIFONE_CVV_ON_PAYMENT');
        Configuration::deleteByName('VERIFONE_CURL_VERIFY_SSL');
        Configuration::deleteByName('VERIFONE_OVERLAY_ON_PLACE_ORDER');

        return $this->unregisterHook('displayHeader')
            && $this->unregisterHook('paymentOptions')
            && $this->unregisterHook('displayOrderConfirmation')
            && $this->unregisterHook('actionProductCancel')
            && $this->unregisterHook('displayAdminOrder')
            && $this->unregisterHook('displayAdminOrderTop')
            && parent::uninstall();
    }

    /**
     * show the settings page, also saves and validates the form on submit
     * @return string
     */
    public function getContent()
    {
        $output = null;

        if (Tools::isSubmit('submit' . $this->name)) {
            $this->reset_token();
            $verifone_title = strval(Tools::getValue('VERIFONE_TITLE'));
            $verifone_description = strval(Tools::getValue('VERIFONE_DESCRIPTION'));
            $verifone_region = strval(Tools::getValue('VERIFONE_REGION'));
            $verifone_api_user = strval(Tools::getValue('VERIFONE_API_USER'));
            $verifone_api_key = strval(Tools::getValue('VERIFONE_API_KEY'));
            $verifone_client_id = strval(Tools::getValue('VERIFONE_CLIENT_ID'));
            $verifone_entity_id = strval(Tools::getValue('VERIFONE_ENTITY_ID'));
            $verifone_payment_contract_id = strval(Tools::getValue('VERIFONE_PAYMENT_CONTRACT_ID'));
            $verifone_3ds_contract_id = strval(Tools::getValue('VERIFONE_3DS_CONTRACT_ID'));
            $verifone_test = strval(Tools::getValue('VERIFONE_TEST'));
            $verifone_test_api_user = strval(Tools::getValue('VERIFONE_TEST_API_USER'));
            $verifone_test_api_key = strval(Tools::getValue('VERIFONE_TEST_API_KEY'));
            $verifone_test_entity_id = strval(Tools::getValue('VERIFONE_TEST_ENTITY_ID'));
            $verifone_test_payment_contract_id = strval(Tools::getValue('VERIFONE_TEST_PAYMENT_CONTRACT_ID'));
            $verifone_test_3ds_contract_id = strval(Tools::getValue('VERIFONE_TEST_3DS_CONTRACT_ID'));
            $verifone_3ds_enable = strval(Tools::getValue('VERIFONE_3DS_ENABLE'));
            $verifone_cvv_on_payment = strval(Tools::getValue('VERIFONE_CVV_ON_PAYMENT'));
            $verifone_curl_verify_ssl = strval(Tools::getValue('VERIFONE_CURL_VERIFY_SSL_cvs'));
            $verifone_overlay_on_place_order = strval(Tools::getValue('VERIFONE_OVERLAY_ON_PLACE_ORDER_oopo'
            ));

            Configuration::updateValue('VERIFONE_TITLE', $verifone_title);
            Configuration::updateValue('VERIFONE_DESCRIPTION', $verifone_description);
            Configuration::updateValue('VERIFONE_REGION', $verifone_region);
            Configuration::updateValue('VERIFONE_API_USER', $verifone_api_user);
            Configuration::updateValue('VERIFONE_API_KEY', base64_encode($verifone_api_key));
            Configuration::updateValue('VERIFONE_ENTITY_ID', $verifone_entity_id);
            Configuration::updateValue('VERIFONE_PAYMENT_CONTRACT_ID', $verifone_payment_contract_id);
            Configuration::updateValue('VERIFONE_3DS_CONTRACT_ID', $verifone_3ds_contract_id);
            Configuration::updateValue('VERIFONE_TEST', $verifone_test);
            Configuration::updateValue('VERIFONE_TEST_API_USER', $verifone_test_api_user);
            Configuration::updateValue('VERIFONE_TEST_API_KEY', base64_encode($verifone_test_api_key)
                , true);
            Configuration::updateValue('VERIFONE_TEST_ENTITY_ID', $verifone_test_entity_id);
            Configuration::updateValue('VERIFONE_TEST_PAYMENT_CONTRACT_ID', $verifone_test_payment_contract_id);
            Configuration::updateValue('VERIFONE_TEST_3DS_CONTRACT_ID', $verifone_test_3ds_contract_id);
            Configuration::updateValue('VERIFONE_3DS_ENABLE', $verifone_3ds_enable);
            Configuration::updateValue('VERIFONE_CVV_ON_PAYMENT', $verifone_cvv_on_payment);
            Configuration::updateValue('VERIFONE_CURL_VERIFY_SSL', $verifone_curl_verify_ssl);
            Configuration::updateValue('VERIFONE_OVERLAY_ON_PLACE_ORDER', $verifone_overlay_on_place_order);
            $output .= $this->displayConfirmation($this->l('Settings updated'));
        }

        return $output . $this->displayForm();
    }


    /**
     * creates the form for the module settings (admin area)
     * @return string
     */
    private function displayForm()
    {
        // Get default language
        $defaultLang = (int)Configuration::get('PS_LANG_DEFAULT');

        // Init Fields form array
        $fieldsForm = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                ],
                'input' => [
                    [
                        'type' => 'text',
                        'label' => $this->l('Title'),
                        'name' => 'VERIFONE_TITLE',
                        'size' => 200,
                        'desc' => $this->l('This controls the title of the payment method which the user sees during checkout.'),
                        'default' => $this->l('Verifone Card Payment')
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Description'),
                        'name' => 'VERIFONE_DESCRIPTION',
                        'size' => 200,
                        'desc' => $this->l('This controls the description which the user sees during checkout. 
                                                Payments powered by Verifone ') . '<a href="https://www.verifone.com/" 
        target="_blank">Verifone </a>',
                        'default' => $this->l('Verifone Card Payment')
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Region'),
                        'desc' => $this->l('Select which region you are based in. Contact your Verifone representative if you are not sure.'),
                        'name' => 'VERIFONE_REGION',
                        //'default_value' => (int) $this->context->country->id,
                        'required' => false,
                        'options' => array(
                            'query' => array(
                                array(
                                    'id' => 'EMEA',
                                    'name' => $this->l('EMEA')
                                ),
                                array(
                                    'id' => 'NA',
                                    'name' => $this->l('NA')
                                ),
                                array(
                                    'id' => 'NZAU',
                                    'name' => $this->l('NZAU')
                                ),
                            ),
                            'id' => 'id',
                            'name' => 'name',
                        ),
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Verifone API User'),
                        'name' => 'VERIFONE_API_USER',
                        'size' => 200,
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Verifone API Key'),
                        'name' => 'VERIFONE_API_KEY',
                        'description' => $this->l('Please enter the Api Key'),
                        'size' => 200,
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Entity ID'),
                        'name' => 'VERIFONE_ENTITY_ID',
                        'size' => 200,
                        'desc' => $this->l('This is provided by a Verifone employee.'),
                        'desc_tip' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Payment Contract ID'),
                        'name' => 'VERIFONE_PAYMENT_CONTRACT_ID',
                        'size' => 200,
                        'desc' => $this->l('This is provided to you by a Verifone employee. It can also be found in the Verifone portal by navigating to the Payment Provider Contract menu underneath Admin. From the URL grab the ID formatted like xxx-xxx-xxx-xxxx'),
                        'desc_tip' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('3DS Contract ID'),
                        'name' => 'VERIFONE_3DS_CONTRACT_ID',
                        'size' => 200,
                        'desc' => $this->l('This is provided by a Verifone employee.'),
                        'desc_tip' => true,
                    ],
                    [
                        'type' => 'radio',
                        'label' => $this->l('Test Mode'),
                        'name' => 'VERIFONE_TEST',
                        'class' => 't',
                        'required' => true,
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'yes',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'no',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ]
                        ],
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ]
            ]
        ];

        $testFieldsForm = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Test credentials'),
                ],
                'input' => [
                    [
                        'type' => 'text',
                        'label' => $this->l('Test API User'),
                        'name' => 'VERIFONE_TEST_API_USER',
                        'size' => 200,
                        'desc' => $this->l('Please enter the API User'),
                        'desc_tip' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Test API Key'),
                        'name' => 'VERIFONE_TEST_API_KEY',
                        'size' => 200,
                        'desc' => $this->l('Please enter the API Key'),
                        'desc_tip' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Test Entity ID'),
                        'name' => 'VERIFONE_TEST_ENTITY_ID',
                        'size' => 200,
                        'desc' => $this->l('This is provided by a Verifone employee.'),
                        'desc_tip' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Test Payment Contract ID'),
                        'name' => 'VERIFONE_TEST_PAYMENT_CONTRACT_ID',
                        'size' => 200,
                        'desc' => $this->l('This is provided to you by a Verifone employee.
                     It can also be found in the Verifone portal by navigating to the Payment Provider Contract menu 
                     underneath Admin. From the URL grab the ID formatted like xxx-xxx-xxx-xxxx'),
                        'desc_tip' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Test 3DS Contract ID'),
                        'name' => 'VERIFONE_TEST_3DS_CONTRACT_ID',
                        'size' => 200,
                        'desc' => $this->l('This is provided by a Verifone employee.'),
                        'desc_tip' => true,
                    ]
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ]
            ]
        ];

        $payment_options = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Payment options'),
                ],
                'input' => [
                    [
                        'type' => 'switch',
                        'label' => $this->l('3DS'),
                        'desc' => $this->l('Enables or disables 3D-Secure for all payments.'),
                        'name' => 'VERIFONE_3DS_ENABLE',
                        'required' => false,
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled'),
                            )
                        ),
                    ],
                    [
                        'type' => 'switch',
                        'label' => $this->l('CVV on payment'),
                        'desc' => $this->l('Requires the CVV  when shoppers pay with a card.'),
                        'name' => 'VERIFONE_CVV_ON_PAYMENT',
                        'required' => false,
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled'),
                            )
                        ),
                    ]
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ]
            ]
        ];
        $curl_verify_ssl = array(array('id' => 'cvs', 'name' => '', 'val' => '1'));
        $overlay_on_place_order = array(array('id' => 'oopo', 'name' => '', 'val' => '1'));
        $advanced_settings = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Advanced options'),
                ],
                'input' => [
                    array(
                        'type' => 'checkbox',
                        'label' => $this->l('Enable/Disable cURL verify host and peer'),
                        'name' => 'VERIFONE_CURL_VERIFY_SSL',
                        'desc' => $this->l('IMPORTANT! If you make requests from a server without SSL please deactivate this.'),
                        'values' => array(
                            'query' => $curl_verify_ssl,
                            'id' => 'id',
                            'name' => 'name',
                            'value' => '1'
                        )
                    ),
                    array(
                        'type' => 'checkbox',
                        'label' => $this->l('Enable/Disable overlay when placing order'),
                        'desc' => $this->l('This option will enable an overlay when placing an order. 
                        This is nothing more than a visual indication for the user that the request is starting'),
                        'name' => 'VERIFONE_OVERLAY_ON_PLACE_ORDER',
                        'values' => array(
                            'query' => $overlay_on_place_order,
                            'id' => 'id',
                            'name' => 'name',
                            'value' => '1'
                        )
                    )
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ]
            ]
        ];

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;

        // Language
        $helper->default_form_language = $defaultLang;
        $helper->allow_employee_form_lang = $defaultLang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;
        $helper->toolbar_scroll = true;
        $helper->submit_action = 'submit' . $this->name;
        $helper->toolbar_btn = [
            'save' => [
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
                    '&token=' . Tools::getAdminTokenLite('AdminModules'),
            ],
            'back' => [
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list'),
            ],
        ];

        // Load current value
        $helper->fields_value['VERIFONE_TITLE'] = Configuration::get('VERIFONE_TITLE');
        $helper->fields_value['VERIFONE_DESCRIPTION'] = Configuration::get('VERIFONE_DESCRIPTION');
        $helper->fields_value['VERIFONE_REGION'] = Configuration::get('VERIFONE_REGION');
        $helper->fields_value['VERIFONE_API_USER'] = Configuration::get('VERIFONE_API_USER');
        $helper->fields_value['VERIFONE_API_KEY'] = base64_decode(Configuration::get('VERIFONE_API_KEY'));
        $helper->fields_value['VERIFONE_ENTITY_ID'] = Configuration::get('VERIFONE_ENTITY_ID');
        $helper->fields_value['VERIFONE_PAYMENT_CONTRACT_ID'] = Configuration::get('VERIFONE_PAYMENT_CONTRACT_ID');
        $helper->fields_value['VERIFONE_3DS_CONTRACT_ID'] = Configuration::get('VERIFONE_3DS_CONTRACT_ID');
        $helper->fields_value['VERIFONE_TEST'] = Configuration::get('VERIFONE_TEST');
        $helper->fields_value['VERIFONE_TEST_API_USER'] = Configuration::get('VERIFONE_TEST_API_USER');
        $helper->fields_value['VERIFONE_TEST_API_KEY'] = base64_decode(Configuration::get('VERIFONE_TEST_API_KEY'));
        $helper->fields_value['VERIFONE_TEST_ENTITY_ID'] = Configuration::get('VERIFONE_TEST_ENTITY_ID');
        $helper->fields_value['VERIFONE_TEST_PAYMENT_CONTRACT_ID'] = Configuration::get('VERIFONE_TEST_PAYMENT_CONTRACT_ID');
        $helper->fields_value['VERIFONE_TEST_3DS_CONTRACT_ID'] = Configuration::get('VERIFONE_TEST_3DS_CONTRACT_ID');
        $helper->fields_value['VERIFONE_3DS_ENABLE'] = Configuration::get('VERIFONE_3DS_ENABLE');
        $helper->fields_value['VERIFONE_CVV_ON_PAYMENT'] = Configuration::get('VERIFONE_CVV_ON_PAYMENT');
        $helper->fields_value['VERIFONE_CURL_VERIFY_SSL_cvs'] = Configuration::get('VERIFONE_CURL_VERIFY_SSL');
        $helper->fields_value['VERIFONE_OVERLAY_ON_PLACE_ORDER_oopo'] = Configuration::get('VERIFONE_OVERLAY_ON_PLACE_ORDER');

        return $helper->generateForm([$fieldsForm, $testFieldsForm, $payment_options, $advanced_settings]);
    }

    /**
     * ==============================HOOKS==============================
     */

    public function hookHeader()
    {
        $returnContent = '';

        $allValues = Tools::getAllValues();
        if (Tools::getValue('controller') == "order") {
            if (!$this->checkActiveModule()) {
                return;
            }

            $link = $this->context->link->getModuleLink($this->name, 'generateUrl',
                ['module_name' => $this->name], true);
            Media::addJsDef(array(
                'verifone_verify_url' => $link
            ));

            $this->context->controller->registerJavascript($this->name . '-gen-url-context',
                'modules/' . $this->name . '/views/assets/js/validate.js'
            );
            $resources[] = _PS_BASE_URL_ . __PS_BASE_URI__ . 'modules/' . $this->name . '/views/assets/js/validate.js' . '?v=' . $this->version;
            $this->context->smarty->assign('resources', $resources);
            $returnContent .= $this->context->smarty->fetch('module:verifone/views/templates/front/prefetch.tpl');
        }

        return $returnContent;
    }

    /**
     * @param $params
     *
     * @return PaymentOption[]|void|void[]
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active || !$this->checkCurrency($params['cart']) || $this->name != 'verifone') {
            return;
        }
        // we clear the cache for every change we make
        Tools::clearSmartyCache();
        Tools::clearXMLCache();
        Media::clearCache();
        Tools::generateIndex();

        return [$this->getVerifonePaymentOption()];
    }

    /**
     * @param $params
     *
     * @return mixed
     */
    public function hookDisplayOrderConfirmation($params)
    {
        if ((!isset($params['order']) || $params['order']->module != $this->name) || !$this->active) {
            return false;
        }

        $order = $params['order'] ?? null;
        $cart = $params['cart'] ?? null;
        $this->smarty->assign(['order' => $order, 'cart' => $cart]);

        return $this->fetch('module:verifone/views/templates/hook/payment_return.tpl');
    }

    public function hookDisplayAdminOrderTop($params)
    {
        $id_order = isset($params['id_order']) ? $params['id_order'] : null;
        $order = new Order((int)$id_order);
        if (Validate::isLoadedObject($order) && $order->module === $this->name) {
            $errors = $this->getAdminOrderPageRefundErrors($params);

            if (!empty($errors)) {
                $this->context->controller->errors[] = $errors;
            }

            $return = $this->disablePartialRefund();
            $return .= $this->getRefundCommentBox();
            $return .= $this->getTotalItemsForRefund($order);

            return $return;
        }
    }

    public function hookDisplayAdminOrder($params)
    {
        if (version_compare(_PS_VERSION_, '1.7.7', '>=')) {
            return false;
        }
        $id_order = isset($params['id_order']) ? $params['id_order'] : null;
        $order = new Order((int)$id_order);
        if (Validate::isLoadedObject($order) && $order->module === $this->name) {

            $errors = $this->getAdminOrderPageRefundErrors($params);

            if (!empty($errors)) {
                $this->context->controller->errors[] = $errors;
            }
            $return = $this->disablePartialRefund();
            $return .= $this->getRefundCommentBox();
            $return .= $this->getTotalItemsForRefund($order);

            return $return;
        }
    }


    /** REFUND
     *
     * @param $orderReturn
     */
    public function hookActionProductCancel($params)
    {

        $order = isset($params['order']) ? $params['order'] : null;
        try {
            if (Validate::isLoadedObject($order) && $order->module === $this->name) {
                if ($order->getCurrentOrderState()->id != Configuration::get('PS_OS_REFUND')) {
                    $orderPayment = OrderPayment::getByOrderReference($order->reference);
                    $transaction_id = null;
                    foreach ($orderPayment as $payment) {
                        $invoice = $payment->getOrderInvoice($order->id);
                        if ($invoice) {
                            $transaction_id = trim($payment->transaction_id);
                        }
                    }

                    //Prepare comment and get only the 150 accepted characters.
                    $refundCommentStr = Tools::getValue('vrefund-comment', '');
                    $refundComment = trim(substr($refundCommentStr, 0, 150));

                    if (!$transaction_id) {
                        $this->logger->error($this->l('Cannot find a valid transaction ID for this order'),
                            __LINE__);
                        $this->context->controller->errors[] = $this->l('Payment module encountered 
                    an error. Cannot get transaction id.');
                    }

                    $refund_class = new Verifone_Refund($order->total_paid, $refundComment,
                        $transaction_id, $order);

                    $response = $this->client->postRefund($refund_class->toArray(), $transaction_id);


                    if (!isset($response['id'])
                        || !isset($response['status'])
                        || 'REFUNDED' !== $response['status']
                    ) {
                        $error_msg = sprintf($this->l('Error Refunding Invoice for order "%s" with 
                    transaction id "%s". Response: %s'),
                            $order->reference, $transaction_id, print_r($response, true));

                        $this->logger->error($error_msg, __LINE__);
                        $this->context->controller->errors[] = $error_msg;
                    } elseif ($response['status'] === 'REFUNDED') {
                        $order->setCurrentState(Configuration::get('PS_OS_REFUND'));
                    }

                    $order->save();
                }
            }
        } catch (Exception $e) {
            $error_msg = sprintf($this->l('Exception, fatal error when trying to refund over Verifone API. Exception: %s'),
                $e->getMessage());
            $this->logger->error($error_msg, __LINE__);
            $this->context->controller->errors[] = $this->l('Fatal error when trying to do refund. 
            You need to initiate a full refund from Verifone portal.');
        }
    }

    /**
     * =============================END HOOKS==============================
     */

    private function getRefundCommentBox()
    {
        $this->context->smarty->assign(
            [
                'verifone_refund_comment_box' => $this->l('Refund Comment...'),
                'verifone_refund_max_length_str' => $this->l('*Max 150 characters.'),
                'verifone_refund_max_length' => 150
            ]);

        if (version_compare(_PS_VERSION_, '1.7.7', '>=')) {
            return $this->context->smarty->fetch('module:verifone/views/templates/hook/refundCommentBox17x.tpl');

        }
        return $this->context->smarty->fetch('module:verifone/views/templates/hook/refundCommentBox.tpl');
    }

    private function getTotalItemsForRefund($order)
    {
        $products = $order->getProducts();
        $prods_array = [];
        foreach ($products as $prod_id => $params) {
            $prods_array[$prod_id] = $params['product_quantity'];
        }
        $notAllProductsAlert = $this->l('Please select all products!');
        $hasPaidShipping = $this->hasPaidShipping($order->getShipping());
        $shippingMsgAlert = $this->l('Order does not have free shipping. Please include that in full refund!');
        $this->context->smarty->assign(
            [
                'verifone_refund_products_list' => json_encode($prods_array),
                'not_all_products_alert' => $notAllProductsAlert,
                'has_paid_shipping' => $hasPaidShipping,
                'shipping_msg_alert' => $shippingMsgAlert
            ]);

        if (version_compare(_PS_VERSION_, '1.7.7', '>=')) {
            return $this->context->smarty->fetch('module:verifone/views/templates/hook/refundItemsCorrection17x.tpl');
        }
        return $this->context->smarty->fetch('module:verifone/views/templates/hook/refundItemsCorrection.tpl');
    }

    private function getAdminOrderPageRefundErrors($params)
    {
        $verifone_msg = '';
        if (isset($this->context->controller->errors['verifone_refund_error'])) {
            $verifone_msg = $this->context->controller->errors['verifone_refund_error'];
        }

        return $verifone_msg;
    }

    private function disablePartialRefund()
    {

        if (version_compare(_PS_VERSION_, '1.7.7', '>=')) {
            return $this->context->smarty->fetch('module:verifone/views/templates/hook/disablePartialRefund17x.tpl');
        }
        return $this->context->smarty->fetch('module:verifone/views/templates/hook/disablePartialRefund.tpl');

    }


    /**
     * @param $cart
     *
     * @return bool
     */
    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }

        return false;
    }

    public function getVerifonePaymentOption()
    {
        if (!$this->active) {
            return;
        }

        //We can bring multiple Verifone Gateway payment options
        $newOption = new PaymentOption();
        $newOption->setCallToActionText($this->l('Pay with Verifone'))
            ->setAction('javascript:VerifoneHostCheckout()');

        return $newOption;
    }

    /**
     * generates the form for the payment option (hosted & inline)
     * @return string
     * @throws SmartyException
     */
    protected function generateForm()
    {
        return $this->context->smarty->fetch('module:verifone/views/templates/hook/payment_options.tpl');
    }

    /**
     * Resets the token on ANY setting change
     */
    public function reset_token()
    {
        $token = Configuration::get('__verifone_api_token');
        if (false !== $token) {
            Configuration::deleteByName('__verifone_api_token');
        }
    }

    public function checkActiveModule()
    {
        $active = false;
        $modules = Hook::getHookModuleExecList('paymentOptions');
        if (empty($modules)) {
            return;
        }
        foreach ($modules as $module) {
            if ($module['module'] == $this->name) {
                $active = true;
            }
        }
        return $active;
    }

    public function hasPaidShipping($shippingArr)
    {
        $total = 0;
        foreach ($shippingArr as $shipping => $valuesArr) {
            $total += $valuesArr['shipping_cost_tax_incl'];
        }
        return $total > 0;
    }

}
